@@ -42,6 +42,8 @@ func TestLoadConfig(t *testing.T) {
 	assert.Equal("side", cfg.GetString("paginatePath"))
 	// default
 	assert.Equal("layouts", cfg.GetString("layoutDir"))
+	// no themes
+	assert.False(cfg.IsSet("allThemes"))
 }
 
 func TestLoadMultiConfig(t *testing.T) {
@@ -95,7 +97,7 @@ top = "top"
 
 [mediaTypes]
 [mediaTypes."text/m1"]
-suffix = "m1main"
+suffixes = ["m1main"]
 
 [outputFormats.o1]
 mediaType = "text/m1"
@@ -133,9 +135,9 @@ p3 = "p3 theme"
 
 [mediaTypes]
 [mediaTypes."text/m1"]
-suffix = "m1theme"
+suffixes = ["m1theme"]
 [mediaTypes."text/m2"]
-suffix = "m2theme"
+suffixes = ["m2theme"]
 
 [outputFormats.o1]
 mediaType = "text/m1"
@@ -205,10 +207,14 @@ map[string]interface {}{
 	b.AssertObject(`
 map[string]interface {}{
   "text/m1": map[string]interface {}{
-    "suffix": "m1main",
+    "suffixes": []interface {}{
+      "m1main",
+    },
   },
   "text/m2": map[string]interface {}{
-    "suffix": "m2theme",
+    "suffixes": []interface {}{
+      "m2theme",
+    },
   },
 }`, got["mediatypes"])
 
@@ -219,8 +225,10 @@ map[string]interface {}{
     "mediatype": Type{
       MainType: "text",
       SubType: "m1",
-      Suffix: "m1main",
       Delimiter: ".",
+      Suffixes: []string{
+        "m1main",
+      },
     },
   },
   "o2": map[string]interface {}{
@@ -228,8 +236,10 @@ map[string]interface {}{
     "mediatype": Type{
       MainType: "text",
       SubType: "m2",
-      Suffix: "m2theme",
       Delimiter: ".",
+      Suffixes: []string{
+        "m2theme",
+      },
     },
   },
 }`, got["outputformats"])
@@ -384,6 +394,6 @@ privacyEnhanced = true
 	b.WithConfigFile("toml", tomlConfig)
 	b.Build(BuildCfg{SkipRender: true})
 
-	assert.True(b.H.Sites[0].Info.Config.Privacy.YouTube.PrivacyEnhanced)
+	assert.True(b.H.Sites[0].Info.Config().Privacy.YouTube.PrivacyEnhanced)
 
 }
