@@ -18,11 +18,12 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"math/rand"
 	"reflect"
 
 	"github.com/gohugoio/hugo/common/maps"
+	"github.com/gohugoio/hugo/common/urls"
 	"github.com/gohugoio/hugo/media"
-	_errors "github.com/pkg/errors"
 
 	"github.com/gohugoio/hugo/langs"
 
@@ -67,8 +68,16 @@ var (
 
 	// Assert that it implements the interface needed for related searches.
 	_ related.Document = (*Page)(nil)
+
+	// Page supports ref and relref
+	_ urls.RefLinker = (*Page)(nil)
 )
 
+// Wraps a Page.
+type pageContainer interface {
+	page() *Page
+}
+
 const (
 	KindPage = "page"
 
@@ -290,17 +299,24 @@ func (p *Page) initContent() {
 		defer cancel()
 		c := make(chan error, 1)
 
+		p.contentInitMu.Lock()
+		defer p.contentInitMu.Unlock()
+
 		go func() {
 			var err error
-			p.contentInitMu.Lock()
-			defer p.contentInitMu.Unlock()
 
-			err = p.prepareForRender()
+			err = p.prepareContent()
 			if err != nil {
 				c <- err
 				return
 			}
 
+			select {
+			case <-ctx.Done():
+				return
+			default:
+			}
+
 			if len(p.summary) == 0 {
 				if err = p.setAutoSummary(); err != nil {
 					err = p.errorf(err, "failed to set auto summary")
@@ -311,7 +327,7 @@ func (p *Page) initContent() {
 
 		select {
 		case <-ctx.Done():
-			p.s.Log.WARN.Printf("WARNING: Timed out creating content for page %q (.Content will be empty). This is most likely a circular shortcode content loop that should be fixed. If this is just a shortcode calling a slow remote service, try to set \"timeout=20000\" (or higher, value is in milliseconds) in config.toml.\n", p.pathOrTitle())
+			p.s.Log.WARN.Printf("Timed out creating content for page %q (.Content will be empty). This is most likely a circular shortcode content loop that should be fixed. If this is just a shortcode calling a slow remote service, try to set \"timeout=30000\" (or higher, value is in milliseconds) in config.toml.\n", p.pathOrTitle())
 		case err := <-c:
 			if err != nil {
 				p.s.SendError(err)
@@ -502,6 +518,14 @@ func (ps Pages) String() string {
 	return fmt.Sprintf("Pages(%d)", len(ps))
 }
 
+// Used in tests.
+func (ps Pages) shuffle() {
+	for i := range ps {
+		j := rand.Intn(i + 1)
+		ps[i], ps[j] = ps[j], ps[i]
+	}
+}
+
 func (ps Pages) findPagePosByFilename(filename string) int {
 	for i, x := range ps {
 		if x.Filename() == filename {
@@ -723,56 +747,48 @@ func splitUserDefinedSummaryAndContent(markup string, c []byte) (sc *summaryCont
 		}
 	}()
 
-	c = bytes.TrimSpace(c)
-	startDivider := bytes.Index(c, internalSummaryDivider)
+	startDivider := bytes.Index(c, internalSummaryDividerBaseBytes)
 
 	if startDivider == -1 {
 		return
 	}
 
-	endDivider := startDivider + len(internalSummaryDivider)
-	endSummary := startDivider
+	startTag := "p"
+	switch markup {
+	case "asciidoc":
+		startTag = "div"
 
-	var (
-		startMarkup []byte
-		endMarkup   []byte
-		addDiv      bool
-	)
+	}
+
+	// Walk back and forward to the surrounding tags.
+	start := bytes.LastIndex(c[:startDivider], []byte("<"+startTag))
+	end := bytes.Index(c[startDivider:], []byte("</"+startTag))
+
+	if start == -1 {
+		start = startDivider
+	} else {
+		start = startDivider - (startDivider - start)
+	}
+
+	if end == -1 {
+		end = startDivider + len(internalSummaryDividerBase)
+	} else {
+		end = startDivider + end + len(startTag) + 3
+	}
+
+	var addDiv bool
 
 	switch markup {
-	default:
-		startMarkup = []byte("<p>")
-		endMarkup = []byte("</p>")
-	case "asciidoc":
-		startMarkup = []byte("<div class=\"paragraph\">")
-		endMarkup = []byte("</div>")
 	case "rst":
-		startMarkup = []byte("<p>")
-		endMarkup = []byte("</p>")
 		addDiv = true
 	}
 
-	// Find the closest end/start markup string to the divider
-	fromStart := -1
-	fromIdx := bytes.LastIndex(c[:startDivider], startMarkup)
-	if fromIdx != -1 {
-		fromStart = startDivider - fromIdx - len(startMarkup)
-	}
-	fromEnd := bytes.Index(c[endDivider:], endMarkup)
+	withoutDivider := append(c[:start], bytes.Trim(c[end:], "\n")...)
 
-	if fromEnd != -1 && fromEnd <= fromStart {
-		endSummary = startDivider + fromEnd + len(endMarkup)
-	} else if fromStart != -1 && fromEnd != -1 {
-		endSummary = startDivider - fromStart - len(startMarkup)
-	}
-
-	withoutDivider := bytes.TrimSpace(append(c[:startDivider], c[endDivider:]...))
-	var (
-		summary []byte
-	)
+	var summary []byte
 
 	if len(withoutDivider) > 0 {
-		summary = bytes.TrimSpace(withoutDivider[:endSummary])
+		summary = bytes.TrimSpace(withoutDivider[:start])
 	}
 
 	if addDiv {
@@ -786,7 +802,7 @@ func splitUserDefinedSummaryAndContent(markup string, c []byte) (sc *summaryCont
 
 	sc = &summaryContent{
 		summary: summary,
-		content: withoutDivider,
+		content: bytes.TrimSpace(withoutDivider),
 	}
 
 	return
@@ -1135,11 +1151,17 @@ func (p *Page) subResourceTargetPathFactory(base string) string {
 	return path.Join(p.relTargetPathBase, base)
 }
 
-func (p *Page) initMainOutputFormat() error {
-	if p.mainPageOutput != nil {
-		return nil
+// Prepare this page for rendering for a new site. The flag start is set
+// for the first site and output format.
+func (p *Page) prepareForRender(start bool) error {
+	p.setContentInit(start)
+	if start {
+		return p.initMainOutputFormat()
 	}
+	return nil
+}
 
+func (p *Page) initMainOutputFormat() error {
 	outFormat := p.outputFormats[0]
 	pageOutput, err := newPageOutput(p, false, false, outFormat)
 
@@ -1186,7 +1208,7 @@ func (p *Page) setContentInit(start bool) error {
 
 }
 
-func (p *Page) prepareForRender() error {
+func (p *Page) prepareContent() error {
 	s := p.s
 
 	// If we got this far it means that this is either a new Page pointer
@@ -1449,7 +1471,7 @@ func (p *Page) updateMetaData(frontmatter map[string]interface{}) error {
 
 	if draft != nil && published != nil {
 		p.Draft = *draft
-		p.s.Log.WARN.Printf("page %q has both draft and published settings in its frontmatter. Using draft.", p.File.Path())
+		p.s.Log.WARN.Printf("page %q has both draft and published settings in its frontmatter. Using draft.", p.Filename())
 	} else if draft != nil {
 		p.Draft = *draft
 	} else if published != nil {
@@ -1864,79 +1886,6 @@ func (p *Page) GetPage(ref string) (*Page, error) {
 	return p.s.getPageNew(p, ref)
 }
 
-type refArgs struct {
-	Path         string
-	Lang         string
-	OutputFormat string
-}
-
-func (p *Page) decodeRefArgs(args map[string]interface{}) (refArgs, *SiteInfo, error) {
-	var ra refArgs
-	err := mapstructure.WeakDecode(args, &ra)
-	if err != nil {
-		return ra, nil, nil
-	}
-	s := p.Site
-
-	if ra.Lang != "" && ra.Lang != p.Lang() {
-		// Find correct site
-		found := false
-		for _, ss := range p.s.owner.Sites {
-			if ss.Lang() == ra.Lang {
-				found = true
-				s = &ss.Info
-			}
-		}
-
-		if !found {
-			p.s.siteRefLinker.logNotFound(ra.Path, fmt.Sprintf("no site found with lang %q", ra.Lang), p)
-			return ra, nil, nil
-		}
-	}
-
-	return ra, s, nil
-}
-
-func (p *Page) Ref(argsm map[string]interface{}) (string, error) {
-	args, s, err := p.decodeRefArgs(argsm)
-	if err != nil {
-		return "", _errors.Wrap(err, "invalid arguments to Ref")
-	}
-
-	if s == nil {
-		return p.s.siteRefLinker.notFoundURL, nil
-	}
-
-	if args.Path == "" {
-		return "", nil
-	}
-
-	if args.OutputFormat != "" {
-		return s.Ref(args.Path, p, args.OutputFormat)
-	}
-	return s.Ref(args.Path, p)
-}
-
-func (p *Page) RelRef(argsm map[string]interface{}) (string, error) {
-	args, s, err := p.decodeRefArgs(argsm)
-	if err != nil {
-		return "", _errors.Wrap(err, "invalid arguments to Ref")
-	}
-
-	if s == nil {
-		return p.s.siteRefLinker.notFoundURL, nil
-	}
-
-	if args.Path == "" {
-		return "", nil
-	}
-
-	if args.OutputFormat != "" {
-		return s.RelRef(args.Path, p, args.OutputFormat)
-	}
-	return s.RelRef(args.Path, p)
-}
-
 func (p *Page) String() string {
 	if sourceRef := p.absoluteSourceRef(); sourceRef != "" {
 		return fmt.Sprintf("Page(%s)", sourceRef)
@@ -2039,8 +1988,6 @@ func (p *Page) initLanguage() {
 		language := ml.Language(p.lang)
 
 		if language == nil {
-			// It can be a file named stefano.chiodino.md.
-			p.s.Log.WARN.Printf("Page language (if it is that) not found in multilang setup: %s.", p.lang)
 			language = ml.DefaultLang
 		}
 
