@@ -16,44 +16,33 @@ package hugolib
 import (
 	"strings"
 	"testing"
-
-	"github.com/spf13/hugo/helpers"
 )
 
 // testdataPermalinks is used by a couple of tests; the expandsTo content is
-// subject to the data in SIMPLE_PAGE_JSON.
+// subject to the data in simplePageJSON.
 var testdataPermalinks = []struct {
 	spec      string
 	valid     bool
 	expandsTo string
 }{
-	//{"/:year/:month/:title/", true, "/2012/04/spf13-vim-3.0-release-and-new-website/"},
-	//{"/:title", true, "/spf13-vim-3.0-release-and-new-website"},
-	//{":title", true, "spf13-vim-3.0-release-and-new-website"},
-	//{"/blog/:year/:yearday/:title", true, "/blog/2012/97/spf13-vim-3.0-release-and-new-website"},
+	{":title", true, "spf13-vim-3.0-release-and-new-website"},
 	{"/:year-:month-:title", true, "/2012-04-spf13-vim-3.0-release-and-new-website"},
-	{"/blog/:year-:month-:title", true, "/blog/2012-04-spf13-vim-3.0-release-and-new-website"},
-	{"/blog-:year-:month-:title", true, "/blog-2012-04-spf13-vim-3.0-release-and-new-website"},
-	//{"/blog/:fred", false, ""},
-	//{"/:year//:title", false, ""},
-	//{
-	//"/:section/:year/:month/:day/:weekdayname/:yearday/:title",
-	//true,
-	//"/blue/2012/04/06/Friday/97/spf13-vim-3.0-release-and-new-website",
-	//},
-	//{
-	//"/:weekday/:weekdayname/:month/:monthname",
-	//true,
-	//"/5/Friday/04/April",
-	//},
-	//{
-	//"/:slug/:title",
-	//true,
-	//"/spf13-vim-3-0-release-and-new-website/spf13-vim-3.0-release-and-new-website",
-	//},
+
+	{"/:year/:yearday/:month/:monthname/:day/:weekday/:weekdayname/", true, "/2012/97/04/April/06/5/Friday/"}, // Dates
+	{"/:section/", true, "/blue/"},                                // Section
+	{"/:title/", true, "/spf13-vim-3.0-release-and-new-website/"}, // Title
+	{"/:slug/", true, "/spf13-vim-3-0-release-and-new-website/"},  // Slug
+	{"/:filename/", true, "/test-page/"},                          // Filename
+	// TODO(moorereason): need test scaffolding for this.
+	//{"/:sections/", false, "/blue/"},                              // Sections
+
+	// Failures
+	{"/blog/:fred", false, ""},
+	{"/:year//:title", false, ""},
 }
 
 func TestPermalinkValidation(t *testing.T) {
+	t.Parallel()
 	for _, item := range testdataPermalinks {
 		pp := pathPattern(item.spec)
 		have := pp.validate()
@@ -71,11 +60,12 @@ func TestPermalinkValidation(t *testing.T) {
 }
 
 func TestPermalinkExpansion(t *testing.T) {
-	page, err := NewPageFrom(strings.NewReader(simplePageJSON), "blue/test-page.md")
-	info := newSiteInfo(siteBuilderCfg{language: helpers.NewDefaultLanguage()})
-	page.Site = &info
+	t.Parallel()
+	s := newTestSite(t)
+	page, err := s.newPageFrom(strings.NewReader(simplePageJSON), "blue/test-page.md")
+
 	if err != nil {
-		t.Fatalf("failed before we began, could not parse SIMPLE_PAGE_JSON: %s", err)
+		t.Fatalf("failed before we began, could not parse simplePageJSON: %s", err)
 	}
 	for _, item := range testdataPermalinks {
 		if !item.valid {
