@@ -19,6 +19,8 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/pkg/errors"
+
 	"github.com/gohugoio/hugo/output"
 )
 
@@ -30,7 +32,7 @@ func (s *Site) renderPages(cfg *BuildCfg) error {
 	pages := make(chan *Page)
 	errs := make(chan error)
 
-	go errorCollator(results, errs)
+	go s.errorCollator(results, errs)
 
 	numWorkers := getGoMaxProcs() * 4
 
@@ -41,7 +43,7 @@ func (s *Site) renderPages(cfg *BuildCfg) error {
 		go pageRenderer(s, pages, results, wg)
 	}
 
-	if len(s.headlessPages) > 0 {
+	if !cfg.PartialReRender && len(s.headlessPages) > 0 {
 		wg.Add(1)
 		go headlessPagesPublisher(s, wg)
 	}
@@ -60,7 +62,7 @@ func (s *Site) renderPages(cfg *BuildCfg) error {
 
 	err := <-errs
 	if err != nil {
-		return fmt.Errorf("Error(s) rendering pages: %s", err)
+		return errors.Wrap(err, "failed to render pages")
 	}
 	return nil
 }
@@ -69,11 +71,11 @@ func headlessPagesPublisher(s *Site, wg *sync.WaitGroup) {
 	defer wg.Done()
 	for _, page := range s.headlessPages {
 		outFormat := page.outputFormats[0] // There is only one
-		if outFormat != s.rc.Format {
+		if outFormat.Name != s.rc.Format.Name {
 			// Avoid double work.
 			continue
 		}
-		pageOutput, err := newPageOutput(page, false, outFormat)
+		pageOutput, err := newPageOutput(page, false, false, outFormat)
 		if err == nil {
 			page.mainPageOutput = pageOutput
 			err = pageOutput.renderResources()
@@ -92,23 +94,20 @@ func pageRenderer(s *Site, pages <-chan *Page, results chan<- error, wg *sync.Wa
 
 		for i, outFormat := range page.outputFormats {
 
+			if outFormat.Name != page.s.rc.Format.Name {
+				// Will be rendered  ... later.
+				continue
+			}
+
 			var (
 				pageOutput *PageOutput
 				err        error
 			)
 
 			if i == 0 {
-				pageOutput, err = newPageOutput(page, false, outFormat)
-				page.mainPageOutput = pageOutput
-			}
-
-			if outFormat != page.s.rc.Format {
-				// Will be rendered  ... later.
-				continue
-			}
-
-			if pageOutput == nil {
-				pageOutput, err = page.mainPageOutput.copyWithFormat(outFormat)
+				pageOutput = page.mainPageOutput
+			} else {
+				pageOutput, err = page.mainPageOutput.copyWithFormat(outFormat, true)
 			}
 
 			if err != nil {
@@ -116,6 +115,10 @@ func pageRenderer(s *Site, pages <-chan *Page, results chan<- error, wg *sync.Wa
 				continue
 			}
 
+			if pageOutput == nil {
+				panic("no pageOutput")
+			}
+
 			// We only need to re-publish the resources if the output format is different
 			// from all of the previous (e.g. the "amp" use case).
 			shouldRender := i == 0
@@ -131,7 +134,7 @@ func pageRenderer(s *Site, pages <-chan *Page, results chan<- error, wg *sync.Wa
 
 			if shouldRender {
 				if err := pageOutput.renderResources(); err != nil {
-					s.Log.ERROR.Printf("Failed to render resources for page %q: %s", page, err)
+					s.SendError(page.errorf(err, "failed to render page resources"))
 					continue
 				}
 			}
@@ -143,7 +146,7 @@ func pageRenderer(s *Site, pages <-chan *Page, results chan<- error, wg *sync.Wa
 			} else {
 				layouts, err = s.layouts(pageOutput)
 				if err != nil {
-					s.Log.ERROR.Printf("Failed to resolve layout output %q for page %q: %s", outFormat.Name, page, err)
+					s.Log.ERROR.Printf("Failed to resolve layout for output %q for page %q: %s", outFormat.Name, page, err)
 					continue
 				}
 			}
@@ -167,7 +170,8 @@ func pageRenderer(s *Site, pages <-chan *Page, results chan<- error, wg *sync.Wa
 					results <- err
 				}
 
-				if pageOutput.IsNode() {
+				// Only render paginators for the main output format
+				if i == 0 && pageOutput.IsNode() {
 					if err := s.renderPaginator(pageOutput); err != nil {
 						results <- err
 					}
@@ -193,7 +197,7 @@ func (s *Site) renderPaginator(p *PageOutput) error {
 
 		// TODO(bep) do better
 		link := newOutputFormat(p.Page, p.outputFormat).Permalink()
-		if err := s.writeDestAlias(target, link, nil); err != nil {
+		if err := s.writeDestAlias(target, link, p.outputFormat, nil); err != nil {
 			return err
 		}
 
@@ -246,12 +250,10 @@ func (s *Site) renderRSS(p *PageOutput) error {
 		return nil
 	}
 
-	p.Kind = kindRSS
-
 	limit := s.Cfg.GetInt("rssLimit")
 	if limit >= 0 && len(p.Pages) > limit {
 		p.Pages = p.Pages[:limit]
-		p.Data["Pages"] = p.Pages
+		p.data["Pages"] = p.Pages
 	}
 
 	layouts, err := s.layoutHandler.For(
@@ -278,7 +280,7 @@ func (s *Site) render404() error {
 	p := s.newNodePage(kind404)
 
 	p.title = "404 Page not found"
-	p.Data["Pages"] = s.Pages
+	p.data["Pages"] = s.Pages
 	p.Pages = s.Pages
 	p.URLPath.URL = "404.html"
 
@@ -291,7 +293,7 @@ func (s *Site) render404() error {
 	htmlOut := output.HTMLFormat
 	htmlOut.BaseName = "404"
 
-	pageOutput, err := newPageOutput(p, false, htmlOut)
+	pageOutput, err := newPageOutput(p, false, false, htmlOut)
 	if err != nil {
 		return err
 	}
@@ -325,7 +327,7 @@ func (s *Site) renderSitemap() error {
 	page.Sitemap.Priority = sitemapDefault.Priority
 	page.Sitemap.Filename = sitemapDefault.Filename
 
-	n.Data["Pages"] = pages
+	n.data["Pages"] = pages
 	n.Pages = pages
 
 	// TODO(bep) we have several of these
@@ -368,12 +370,12 @@ func (s *Site) renderRobotsTXT() error {
 	if err := p.initTargetPathDescriptor(); err != nil {
 		return err
 	}
-	p.Data["Pages"] = s.Pages
+	p.data["Pages"] = s.Pages
 	p.Pages = s.Pages
 
 	rLayouts := []string{"robots.txt", "_default/robots.txt", "_internal/_default/robots.txt"}
 
-	pageOutput, err := newPageOutput(p, false, output.RobotsTxtFormat)
+	pageOutput, err := newPageOutput(p, false, false, output.RobotsTxtFormat)
 	if err != nil {
 		return err
 	}
@@ -415,7 +417,7 @@ func (s *Site) renderAliases() error {
 					a = path.Join(lang, a)
 				}
 
-				if err := s.writeDestAlias(a, plink, p); err != nil {
+				if err := s.writeDestAlias(a, plink, f, p); err != nil {
 					return err
 				}
 			}
@@ -423,18 +425,21 @@ func (s *Site) renderAliases() error {
 	}
 
 	if s.owner.multilingual.enabled() && !s.owner.IsMultihost() {
-		mainLang := s.owner.multilingual.DefaultLang
-		if s.Info.defaultContentLanguageInSubdir {
-			mainLangURL := s.PathSpec.AbsURL(mainLang.Lang, false)
-			s.Log.DEBUG.Printf("Write redirect to main language %s: %s", mainLang, mainLangURL)
-			if err := s.publishDestAlias(true, "/", mainLangURL, nil); err != nil {
-				return err
-			}
-		} else {
-			mainLangURL := s.PathSpec.AbsURL("", false)
-			s.Log.DEBUG.Printf("Write redirect to main language %s: %s", mainLang, mainLangURL)
-			if err := s.publishDestAlias(true, mainLang.Lang, mainLangURL, nil); err != nil {
-				return err
+		html, found := s.outputFormatsConfig.GetByName("HTML")
+		if found {
+			mainLang := s.owner.multilingual.DefaultLang
+			if s.Info.defaultContentLanguageInSubdir {
+				mainLangURL := s.PathSpec.AbsURL(mainLang.Lang, false)
+				s.Log.DEBUG.Printf("Write redirect to main language %s: %s", mainLang, mainLangURL)
+				if err := s.publishDestAlias(true, "/", mainLangURL, html, nil); err != nil {
+					return err
+				}
+			} else {
+				mainLangURL := s.PathSpec.AbsURL("", false)
+				s.Log.DEBUG.Printf("Write redirect to main language %s: %s", mainLang, mainLangURL)
+				if err := s.publishDestAlias(true, mainLang.Lang, mainLangURL, html, nil); err != nil {
+					return err
+				}
 			}
 		}
 	}
