@@ -4,7 +4,9 @@ import (
 	"io/ioutil"
 	"path/filepath"
 	"runtime"
+	"strconv"
 	"testing"
+	"unicode/utf8"
 
 	"bytes"
 	"fmt"
@@ -14,7 +16,6 @@ import (
 
 	"github.com/gohugoio/hugo/langs"
 	"github.com/sanity-io/litter"
-	jww "github.com/spf13/jwalterweatherman"
 
 	"github.com/gohugoio/hugo/config"
 	"github.com/gohugoio/hugo/deps"
@@ -26,19 +27,18 @@ import (
 
 	"os"
 
+	"github.com/gohugoio/hugo/common/loggers"
 	"github.com/gohugoio/hugo/hugofs"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
 
-const ()
-
 type sitesBuilder struct {
 	Cfg config.Provider
 	Fs  *hugofs.Fs
 	T   testing.TB
 
-	logger *jww.Notepad
+	logger *loggers.Logger
 
 	dumper litter.Options
 
@@ -103,7 +103,7 @@ func (s *sitesBuilder) Running() *sitesBuilder {
 	return s
 }
 
-func (s *sitesBuilder) WithLogger(logger *jww.Notepad) *sitesBuilder {
+func (s *sitesBuilder) WithLogger(logger *loggers.Logger) *sitesBuilder {
 	s.logger = logger
 	return s
 }
@@ -181,10 +181,13 @@ privacyEnhanced = true
 `
 
 func (s *sitesBuilder) WithSimpleConfigFile() *sitesBuilder {
-	var config = `
-baseURL = "http://example.com/"
+	return s.WithSimpleConfigFileAndBaseURL("http://example.com/")
+}
 
-` + commonConfigSections
+func (s *sitesBuilder) WithSimpleConfigFileAndBaseURL(baseURL string) *sitesBuilder {
+	config := fmt.Sprintf("baseURL = %q", baseURL)
+
+	config = config + commonConfigSections
 	return s.WithConfigFile("toml", config)
 }
 
@@ -312,6 +315,14 @@ func (s *sitesBuilder) writeFilePairs(folder string, filenameContent []string) *
 }
 
 func (s *sitesBuilder) CreateSites() *sitesBuilder {
+	if err := s.CreateSitesE(); err != nil {
+		s.Fatalf("Failed to create sites: %s", err)
+	}
+
+	return s
+}
+
+func (s *sitesBuilder) CreateSitesE() error {
 	s.addDefaults()
 	s.writeFilePairs("content", s.contentFilePairs)
 	s.writeFilePairs("content", s.contentFilePairsAdded)
@@ -325,7 +336,7 @@ func (s *sitesBuilder) CreateSites() *sitesBuilder {
 	if s.Cfg == nil {
 		cfg, _, err := LoadConfig(ConfigSourceDescriptor{Fs: s.Fs.Source, Filename: "config." + s.configFormat})
 		if err != nil {
-			s.Fatalf("Failed to load config: %s", err)
+			return err
 		}
 		// TODO(bep)
 		/*		expectedConfigs := 1
@@ -339,11 +350,19 @@ func (s *sitesBuilder) CreateSites() *sitesBuilder {
 
 	sites, err := NewHugoSites(deps.DepsCfg{Fs: s.Fs, Cfg: s.Cfg, Logger: s.logger, Running: s.running})
 	if err != nil {
-		s.Fatalf("Failed to create sites: %s", err)
+		return err
 	}
 	s.H = sites
 
-	return s
+	return nil
+}
+
+func (s *sitesBuilder) BuildE(cfg BuildCfg) error {
+	if s.H == nil {
+		s.CreateSites()
+	}
+
+	return s.H.Build(cfg)
 }
 
 func (s *sitesBuilder) Build(cfg BuildCfg) *sitesBuilder {
@@ -360,6 +379,7 @@ func (s *sitesBuilder) build(cfg BuildCfg, shouldFail bool) *sitesBuilder {
 	}
 
 	err := s.H.Build(cfg)
+
 	if err == nil {
 		logErrorCount := s.H.NumLogErrors()
 		if logErrorCount > 0 {
@@ -448,12 +468,16 @@ func (s *sitesBuilder) Fatalf(format string, args ...interface{}) {
 }
 
 func Fatalf(t testing.TB, format string, args ...interface{}) {
-	trace := strings.Join(assert.CallerInfo(), "\n\r\t\t\t")
+	trace := trace()
 	format = format + "\n%s"
 	args = append(args, trace)
 	t.Fatalf(format, args...)
 }
 
+func trace() string {
+	return strings.Join(assert.CallerInfo(), "\n\r\t\t\t")
+}
+
 func (s *sitesBuilder) AssertFileContent(filename string, matches ...string) {
 	content := readDestination(s.T, s.Fs, filename)
 	for _, match := range matches {
@@ -639,13 +663,19 @@ func createWithTemplateFromNameValues(additionalTemplates ...string) func(templ
 }
 
 func buildSingleSite(t testing.TB, depsCfg deps.DepsCfg, buildCfg BuildCfg) *Site {
-	return buildSingleSiteExpected(t, false, depsCfg, buildCfg)
+	return buildSingleSiteExpected(t, false, false, depsCfg, buildCfg)
 }
 
-func buildSingleSiteExpected(t testing.TB, expectBuildError bool, depsCfg deps.DepsCfg, buildCfg BuildCfg) *Site {
+func buildSingleSiteExpected(t testing.TB, expectSiteInitEror, expectBuildError bool, depsCfg deps.DepsCfg, buildCfg BuildCfg) *Site {
 	h, err := NewHugoSites(depsCfg)
 
-	require.NoError(t, err)
+	if expectSiteInitEror {
+		require.Error(t, err)
+		return nil
+	} else {
+		require.NoError(t, err)
+	}
+
 	require.Len(t, h.Sites, 1)
 
 	if expectBuildError {
@@ -673,6 +703,26 @@ func dumpPages(pages ...*Page) {
 	}
 }
 
+func printStringIndexes(s string) {
+	lines := strings.Split(s, "\n")
+	i := 0
+
+	for _, line := range lines {
+
+		for _, r := range line {
+			fmt.Printf("%-3s", strconv.Itoa(i))
+			i += utf8.RuneLen(r)
+		}
+		i++
+		fmt.Println()
+		for _, r := range line {
+			fmt.Printf("%-3s", string(r))
+		}
+		fmt.Println()
+
+	}
+
+}
 func isCI() bool {
 	return os.Getenv("CI") != ""
 }
