@@ -32,6 +32,7 @@ type stateFunc func(*pageLexer) stateFunc
 type lexerShortcodeState struct {
 	currLeftDelimItem  ItemType
 	currRightDelimItem ItemType
+	isInline           bool
 	currShortcodeName  string          // is only set when a shortcode is in opened state
 	closingState       int             // > 0 = on its way to be closed
 	elementStepNum     int             // step number in element
@@ -52,6 +53,8 @@ type pageLexer struct {
 	summaryDivider []byte
 	// Set when we have parsed any summary divider
 	summaryDividerChecked bool
+	// Whether we're in a HTML comment.
+	isInHTMLComment bool
 
 	lexerShortcodeState
 
@@ -119,7 +122,7 @@ var (
 	delimYAML         = []byte("---")
 	delimOrg          = []byte("#+")
 	htmlCommentStart  = []byte("<!--")
-	htmlCOmmentEnd    = []byte("-->")
+	htmlCommentEnd    = []byte("-->")
 )
 
 func (l *pageLexer) next() rune {
@@ -194,7 +197,30 @@ func (l *pageLexer) consumeCRLF() bool {
 	return consumed
 }
 
+func (l *pageLexer) consumeToNextLine() {
+	for {
+		r := l.next()
+		if r == eof || isEndOfLine(r) {
+			return
+		}
+	}
+}
+
+func (l *pageLexer) consumeSpace() {
+	for {
+		r := l.next()
+		if r == eof || !unicode.IsSpace(r) {
+			l.backup()
+			return
+		}
+	}
+}
+
 func lexMainSection(l *pageLexer) stateFunc {
+	if l.isInHTMLComment {
+		return lexEndFromtMatterHTMLComment
+	}
+
 	// Fast forward as far as possible.
 	var l1, l2 int
 
@@ -206,7 +232,7 @@ func lexMainSection(l *pageLexer) stateFunc {
 	}
 
 	l2 = l.index(leftDelimSc)
-	skip := minPositiveIndex(l1, l2)
+	skip := minIndex(l1, l2)
 
 	if skip > 0 {
 		l.pos += skip
@@ -214,6 +240,19 @@ func lexMainSection(l *pageLexer) stateFunc {
 
 	for {
 		if l.isShortCodeStart() {
+			if l.isInline {
+				// If we're inside an inline shortcode, the only valid shortcode markup is
+				// the markup which closes it.
+				b := l.input[l.pos+3:]
+				end := indexNonWhiteSpace(b, '/')
+				if end != len(l.input)-1 {
+					b = bytes.TrimSpace(b[end+1:])
+					if end == -1 || !bytes.HasPrefix(b, []byte(l.currShortcodeName+" ")) {
+						return l.errorf("inline shortcodes do not support nesting")
+					}
+				}
+			}
+
 			if l.pos > l.start {
 				l.emit(tText)
 			}
@@ -234,7 +273,12 @@ func lexMainSection(l *pageLexer) stateFunc {
 				}
 				l.summaryDividerChecked = true
 				l.pos += len(l.summaryDivider)
+				// This makes it a little easier to reason about later.
+				l.consumeSpace()
 				l.emit(TypeLeadSummaryDivider)
+
+				// We have already moved to the next.
+				continue
 			}
 		}
 
@@ -251,6 +295,14 @@ func lexMainSection(l *pageLexer) stateFunc {
 
 func (l *pageLexer) isShortCodeStart() bool {
 	return l.hasPrefix(leftDelimScWithMarkup) || l.hasPrefix(leftDelimScNoMarkup)
+
+}
+
+func (l *pageLexer) posFirstNonWhiteSpace() int {
+	f := func(c rune) bool {
+		return !unicode.IsSpace(c)
+	}
+	return bytes.IndexFunc(l.input[l.pos:], f)
 }
 
 func lexIntroSection(l *pageLexer) stateFunc {
@@ -275,16 +327,15 @@ LOOP:
 		case r == byteOrderMark:
 			l.emit(TypeIgnore)
 		case !isSpace(r) && !isEndOfLine(r):
-			// No front matter.
 			if r == '<' {
 				l.backup()
 				if l.hasPrefix(htmlCommentStart) {
-					right := l.index(htmlCOmmentEnd)
-					if right == -1 {
-						return l.errorf("starting HTML comment with no end")
-					}
-					l.pos += right + len(htmlCOmmentEnd)
-					l.emit(TypeHTMLComment)
+					// This may be commented out front mattter, which should
+					// still be read.
+					l.consumeToNextLine()
+					l.isInHTMLComment = true
+					l.emit(TypeIgnore)
+					continue LOOP
 				} else {
 					if l.pos > l.start {
 						l.emit(tText)
@@ -304,6 +355,19 @@ LOOP:
 	return lexMainSection
 }
 
+func lexEndFromtMatterHTMLComment(l *pageLexer) stateFunc {
+	l.isInHTMLComment = false
+	right := l.index(htmlCommentEnd)
+	if right == -1 {
+		return l.errorf("starting HTML comment with no end")
+	}
+	l.pos += right + len(htmlCommentEnd)
+	l.emit(TypeIgnore)
+
+	// Now move on to the shortcodes.
+	return lexMainSection
+}
+
 func lexDone(l *pageLexer) stateFunc {
 
 	// Done!
@@ -596,6 +660,9 @@ Loop:
 	return lexInsideShortcode
 }
 
+// Inline shortcodes has the form {{< myshortcode.inline >}}
+var inlineIdentifier = []byte("inline ")
+
 // scans an alphanumeric inside shortcode
 func lexIdentifierInShortcode(l *pageLexer) stateFunc {
 	lookForEnd := false
@@ -605,6 +672,11 @@ Loop:
 		case isAlphaNumericOrHyphen(r):
 		// Allow forward slash inside names to make it possible to create namespaces.
 		case r == '/':
+		case r == '.':
+			l.isInline = l.hasPrefix(inlineIdentifier)
+			if !l.isInline {
+				return l.errorf("period in shortcode name only allowed for inline identifiers")
+			}
 		default:
 			l.backup()
 			word := string(l.input[l.start:l.pos])
@@ -619,7 +691,11 @@ Loop:
 			l.currShortcodeName = word
 			l.openShortcodes[word] = true
 			l.elementStepNum++
-			l.emit(tScName)
+			if l.isInline {
+				l.emit(tScNameInline)
+			} else {
+				l.emit(tScName)
+			}
 			break Loop
 		}
 	}
@@ -631,6 +707,7 @@ Loop:
 }
 
 func lexEndOfShortcode(l *pageLexer) stateFunc {
+	l.isInline = false
 	if l.hasPrefix(l.currentRightShortcodeDelim()) {
 		return lexShortcodeRightDelim
 	}
@@ -715,12 +792,12 @@ func (l *pageLexer) currentRightShortcodeDelim() []byte {
 
 // helper functions
 
-// returns the min index > 0
-func minPositiveIndex(indices ...int) int {
+// returns the min index >= 0
+func minIndex(indices ...int) int {
 	min := -1
 
 	for _, j := range indices {
-		if j <= 0 {
+		if j < 0 {
 			continue
 		}
 		if min == -1 {
@@ -732,6 +809,22 @@ func minPositiveIndex(indices ...int) int {
 	return min
 }
 
+func indexNonWhiteSpace(s []byte, in rune) int {
+	idx := bytes.IndexFunc(s, func(r rune) bool {
+		return !unicode.IsSpace(r)
+	})
+
+	if idx == -1 {
+		return -1
+	}
+
+	r, _ := utf8.DecodeRune(s[idx:])
+	if r == in {
+		return idx
+	}
+	return -1
+}
+
 func isSpace(r rune) bool {
 	return r == ' ' || r == '\t'
 }
