@@ -16,7 +16,6 @@
 package releaser
 
 import (
-	"errors"
 	"fmt"
 	"io/ioutil"
 	"log"
@@ -26,6 +25,8 @@ import (
 	"regexp"
 	"strings"
 
+	"github.com/pkg/errors"
+
 	"github.com/gohugoio/hugo/helpers"
 )
 
@@ -255,7 +256,7 @@ func (r *ReleaseHandler) release(releaseNotesFile string) error {
 	cmd.Stderr = os.Stderr
 	err := cmd.Run()
 	if err != nil {
-		return fmt.Errorf("goreleaser failed: %s", err)
+		return errors.Wrap(err, "goreleaser failed")
 	}
 	return nil
 }
@@ -278,7 +279,7 @@ func (r *ReleaseHandler) bumpVersions(ver helpers.HugoVersion) error {
 	if ver.Suffix != "" {
 		snapcraftGrade = "devel"
 	}
-	if err := r.replaceInFile("snapcraft.yaml",
+	if err := r.replaceInFile("snap/snapcraft.yaml",
 		`version: "(.*)"`, fmt.Sprintf(`version: "%s"`, ver),
 		`grade: (.*) #`, fmt.Sprintf(`grade: %s #`, snapcraftGrade)); err != nil {
 		return err
