@@ -14,7 +14,6 @@
 package resource
 
 import (
-	"errors"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -25,12 +24,15 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/gohugoio/hugo/output"
 	"github.com/gohugoio/hugo/tpl"
+	"github.com/pkg/errors"
 
+	"github.com/gohugoio/hugo/cache/filecache"
+	"github.com/gohugoio/hugo/common/collections"
+	"github.com/gohugoio/hugo/common/hugio"
 	"github.com/gohugoio/hugo/common/loggers"
 
-	jww "github.com/spf13/jwalterweatherman"
-
 	"github.com/spf13/afero"
 
 	"github.com/gobwas/glob"
@@ -47,10 +49,9 @@ var (
 	_ Cloner                  = (*genericResource)(nil)
 	_ ResourcesLanguageMerger = (*Resources)(nil)
 	_ permalinker             = (*genericResource)(nil)
+	_ collections.Slicer      = (*genericResource)(nil)
 )
 
-const DefaultResourceType = "unknown"
-
 var noData = make(map[string]interface{})
 
 // Source is an internal template and not meant for use in the templates. It
@@ -62,8 +63,8 @@ type Source interface {
 type permalinker interface {
 	relPermalinkFor(target string) string
 	permalinkFor(target string) string
-	relTargetPathFor(target string) string
-	relTargetPath() string
+	relTargetPathsFor(target string) []string
+	relTargetPaths() []string
 	targetPath() string
 }
 
@@ -108,6 +109,8 @@ type Resource interface {
 	Params() map[string]interface{}
 }
 
+// ResourcesLanguageMerger describes an interface for merging resources from a
+// different language.
 type ResourcesLanguageMerger interface {
 	MergeByLanguage(other Resources) Resources
 	// Needed for integration with the tpl package.
@@ -134,28 +137,26 @@ type ContentResource interface {
 	Content() (interface{}, error)
 }
 
-// ReadSeekCloser is implemented by afero.File. We use this as the common type for
-// content in Resource objects, even for strings.
-type ReadSeekCloser interface {
-	io.Reader
-	io.Seeker
-	io.Closer
-}
-
-// OpenReadSeekeCloser allows setting some other way (than reading from a filesystem)
+// OpenReadSeekCloser allows setting some other way (than reading from a filesystem)
 // to open or create a ReadSeekCloser.
-type OpenReadSeekCloser func() (ReadSeekCloser, error)
+type OpenReadSeekCloser func() (hugio.ReadSeekCloser, error)
 
 // ReadSeekCloserResource is a Resource that supports loading its content.
 type ReadSeekCloserResource interface {
 	Resource
-	ReadSeekCloser() (ReadSeekCloser, error)
+	ReadSeekCloser() (hugio.ReadSeekCloser, error)
 }
 
 // Resources represents a slice of resources, which can be a mix of different types.
 // I.e. both pages and images etc.
 type Resources []Resource
 
+// ResourcesConverter converts a given slice of Resource objects to Resources.
+type ResourcesConverter interface {
+	ToResources() Resources
+}
+
+// ByType returns resources of a given resource type (ie. "image").
 func (r Resources) ByType(tp string) Resources {
 	var filtered Resources
 
@@ -236,19 +237,19 @@ func getGlob(pattern string) (glob.Glob, error) {
 }
 
 // MergeByLanguage adds missing translations in r1 from r2.
-func (r1 Resources) MergeByLanguage(r2 Resources) Resources {
-	result := append(Resources(nil), r1...)
+func (r Resources) MergeByLanguage(r2 Resources) Resources {
+	result := append(Resources(nil), r...)
 	m := make(map[string]bool)
-	for _, r := range r1 {
-		if translated, ok := r.(translatedResource); ok {
+	for _, rr := range r {
+		if translated, ok := rr.(translatedResource); ok {
 			m[translated.TranslationKey()] = true
 		}
 	}
 
-	for _, r := range r2 {
-		if translated, ok := r.(translatedResource); ok {
+	for _, rr := range r2 {
+		if translated, ok := rr.(translatedResource); ok {
 			if _, found := m[translated.TranslationKey()]; !found {
-				result = append(result, r)
+				result = append(result, rr)
 			}
 		}
 	}
@@ -257,20 +258,21 @@ func (r1 Resources) MergeByLanguage(r2 Resources) Resources {
 
 // MergeByLanguageInterface is the generic version of MergeByLanguage. It
 // is here just so it can be called from the tpl package.
-func (r1 Resources) MergeByLanguageInterface(in interface{}) (interface{}, error) {
+func (r Resources) MergeByLanguageInterface(in interface{}) (interface{}, error) {
 	r2, ok := in.(Resources)
 	if !ok {
 		return nil, fmt.Errorf("%T cannot be merged by language", in)
 	}
-	return r1.MergeByLanguage(r2), nil
+	return r.MergeByLanguage(r2), nil
 }
 
 type Spec struct {
 	*helpers.PathSpec
 
-	MediaTypes media.Types
+	MediaTypes    media.Types
+	OutputFormats output.Formats
 
-	Logger *jww.Notepad
+	Logger *loggers.Logger
 
 	TextTemplates tpl.TemplateParseFinder
 
@@ -279,12 +281,15 @@ type Spec struct {
 
 	imageCache    *imageCache
 	ResourceCache *ResourceCache
-
-	GenImagePath  string
-	GenAssetsPath string
+	FileCaches    filecache.Caches
 }
 
-func NewSpec(s *helpers.PathSpec, logger *jww.Notepad, mimeTypes media.Types) (*Spec, error) {
+func NewSpec(
+	s *helpers.PathSpec,
+	fileCaches filecache.Caches,
+	logger *loggers.Logger,
+	outputFormats output.Formats,
+	mimeTypes media.Types) (*Spec, error) {
 
 	imaging, err := decodeImaging(s.Cfg.GetStringMap("imaging"))
 	if err != nil {
@@ -295,23 +300,16 @@ func NewSpec(s *helpers.PathSpec, logger *jww.Notepad, mimeTypes media.Types) (*
 		logger = loggers.NewErrorLogger()
 	}
 
-	genImagePath := filepath.FromSlash("_gen/images")
-	// The transformed assets (CSS etc.)
-	genAssetsPath := filepath.FromSlash("_gen/assets")
-
 	rs := &Spec{PathSpec: s,
 		Logger:        logger,
-		GenImagePath:  genImagePath,
-		GenAssetsPath: genAssetsPath,
 		imaging:       &imaging,
 		MediaTypes:    mimeTypes,
+		OutputFormats: outputFormats,
+		FileCaches:    fileCaches,
 		imageCache: newImageCache(
+			fileCaches.ImageCache(),
+
 			s,
-			// We're going to write a cache pruning routine later, so make it extremely
-			// unlikely that the user shoots him or herself in the foot
-			// and this is set to a value that represents data he/she
-			// cares about. This should be set in stone once released.
-			genImagePath,
 		)}
 
 	rs.ResourceCache = newResourceCache(rs)
@@ -338,10 +336,12 @@ type ResourceSourceDescriptor struct {
 	// Typically the language code if this resource should be published to its sub-folder.
 	URLBase string
 
-	// Any base path prepended to the target path. This will also typically be the
+	// Any base paths prepended to the target path. This will also typically be the
 	// language code, but setting it here means that it should not have any effect on
 	// the permalink.
-	TargetPathBase string
+	// This may be several values. In multihost mode we may publish the same resources to
+	// multiple targets.
+	TargetBasePaths []string
 
 	// Delay publishing until either Permalink or RelPermalink is called. Maybe never.
 	LazyPublish bool
@@ -375,18 +375,15 @@ func (r *Spec) newResourceForFs(sourceFs afero.Fs, fd ResourceSourceDescriptor)
 		}
 	}
 
-	if fd.URLBase == "" {
-		fd.URLBase = r.GetURLLanguageBasePath()
-	}
-
-	if fd.TargetPathBase == "" {
-		fd.TargetPathBase = r.GetTargetLanguageBasePath()
-	}
-
 	if fd.RelTargetFilename == "" {
 		fd.RelTargetFilename = fd.Filename()
 	}
 
+	if len(fd.TargetBasePaths) == 0 {
+		// If not set, we publish the same resource to all hosts.
+		fd.TargetBasePaths = r.MultihostTargetBasePaths
+	}
+
 	return r.newResource(sourceFs, fd)
 }
 
@@ -416,16 +413,15 @@ func (r *Spec) newResource(sourceFs afero.Fs, fd ResourceSourceDescriptor) (Reso
 	mimeType, found := r.MediaTypes.GetFirstBySuffix(strings.TrimPrefix(ext, "."))
 	// TODO(bep) we need to handle these ambigous types better, but in this context
 	// we most likely want the application/xml type.
-	if mimeType.Suffix == "xml" && mimeType.SubType == "rss" {
+	if mimeType.Suffix() == "xml" && mimeType.SubType == "rss" {
 		mimeType, found = r.MediaTypes.GetByType("application/xml")
 	}
 
 	if !found {
 		mimeStr := mime.TypeByExtension(ext)
 		if mimeStr != "" {
-			mimeType, _ = media.FromString(mimeStr)
+			mimeType, _ = media.FromStringAndExt(mimeStr, ext)
 		}
-
 	}
 
 	gr := r.newGenericResourceWithBase(
@@ -433,7 +429,7 @@ func (r *Spec) newResource(sourceFs afero.Fs, fd ResourceSourceDescriptor) (Reso
 		fd.LazyPublish,
 		fd.OpenReadSeekCloser,
 		fd.URLBase,
-		fd.TargetPathBase,
+		fd.TargetBasePaths,
 		fd.TargetPathBuilder,
 		fi,
 		sourceFilename,
@@ -520,8 +516,8 @@ type resourcePathDescriptor struct {
 	baseURLDir string
 
 	// This will normally be the same as above, but this will only apply to publishing
-	// of resources.
-	baseTargetPathDir string
+	// of resources. It may be mulltiple values when in multihost mode.
+	baseTargetPathDirs []string
 
 	// baseOffset is set when the output format's path has a offset, e.g. for AMP.
 	baseOffset string
@@ -540,7 +536,7 @@ type resourceHash struct {
 type publishOnce struct {
 	publisherInit sync.Once
 	publisherErr  error
-	logger        *jww.Notepad
+	logger        *loggers.Logger
 }
 
 func (l *publishOnce) publish(s Source) error {
@@ -555,6 +551,7 @@ func (l *publishOnce) publish(s Source) error {
 
 // genericResource represents a generic linkable resource.
 type genericResource struct {
+	commonResource
 	resourcePathDescriptor
 
 	title  string
@@ -591,6 +588,9 @@ type genericResource struct {
 	*publishOnce
 }
 
+type commonResource struct {
+}
+
 func (l *genericResource) Data() interface{} {
 	return noData
 }
@@ -603,7 +603,7 @@ func (l *genericResource) Content() (interface{}, error) {
 	return l.content, nil
 }
 
-func (l *genericResource) ReadSeekCloser() (ReadSeekCloser, error) {
+func (l *genericResource) ReadSeekCloser() (hugio.ReadSeekCloser, error) {
 	if l.openReadSeekerCloser != nil {
 		return l.openReadSeekerCloser()
 	}
@@ -626,14 +626,35 @@ func (l genericResource) WithNewBase(base string) Resource {
 	return &l
 }
 
+// Slice is not meant to be used externally. It's a bridge function
+// for the template functions. See collections.Slice.
+func (commonResource) Slice(in interface{}) (interface{}, error) {
+	switch items := in.(type) {
+	case Resources:
+		return items, nil
+	case []interface{}:
+		groups := make(Resources, len(items))
+		for i, v := range items {
+			g, ok := v.(Resource)
+			if !ok {
+				return nil, fmt.Errorf("type %T is not a Resource", v)
+			}
+			groups[i] = g
+		}
+		return groups, nil
+	default:
+		return nil, fmt.Errorf("invalid slice type %T", items)
+	}
+}
+
 func (l *genericResource) initHash() error {
 	var err error
 	l.hashInit.Do(func() {
 		var hash string
-		var f ReadSeekCloser
+		var f hugio.ReadSeekCloser
 		f, err = l.ReadSeekCloser()
 		if err != nil {
-			err = fmt.Errorf("failed to open source file: %s", err)
+			err = errors.Wrap(err, "failed to open source file")
 			return
 		}
 		defer f.Close()
@@ -652,7 +673,7 @@ func (l *genericResource) initHash() error {
 func (l *genericResource) initContent() error {
 	var err error
 	l.contentInit.Do(func() {
-		var r ReadSeekCloser
+		var r hugio.ReadSeekCloser
 		r, err = l.ReadSeekCloser()
 		if err != nil {
 			return
@@ -687,7 +708,7 @@ func (l *genericResource) publishIfNeeded() {
 
 func (l *genericResource) Permalink() string {
 	l.publishIfNeeded()
-	return l.spec.PermalinkForBaseURL(l.relPermalinkForRel(l.relTargetDirFile.path()), l.spec.BaseURL.HostURL())
+	return l.spec.PermalinkForBaseURL(l.relPermalinkForRel(l.relTargetDirFile.path(), true), l.spec.BaseURL.HostURL())
 }
 
 func (l *genericResource) RelPermalink() string {
@@ -696,19 +717,19 @@ func (l *genericResource) RelPermalink() string {
 }
 
 func (l *genericResource) relPermalinkFor(target string) string {
-	return l.relPermalinkForRel(target)
+	return l.relPermalinkForRel(target, false)
 
 }
 func (l *genericResource) permalinkFor(target string) string {
-	return l.spec.PermalinkForBaseURL(l.relPermalinkForRel(target), l.spec.BaseURL.HostURL())
+	return l.spec.PermalinkForBaseURL(l.relPermalinkForRel(target, true), l.spec.BaseURL.HostURL())
 
 }
-func (l *genericResource) relTargetPathFor(target string) string {
-	return l.relTargetPathForRel(target, false)
+func (l *genericResource) relTargetPathsFor(target string) []string {
+	return l.relTargetPathsForRel(target)
 }
 
-func (l *genericResource) relTargetPath() string {
-	return l.relTargetPathForRel(l.targetPath(), false)
+func (l *genericResource) relTargetPaths() []string {
+	return l.relTargetPathsForRel(l.targetPath())
 }
 
 func (l *genericResource) Name() string {
@@ -745,12 +766,35 @@ func (l *genericResource) updateParams(params map[string]interface{}) {
 	}
 }
 
-func (l *genericResource) relPermalinkForRel(rel string) string {
-	return l.spec.PathSpec.URLizeFilename(l.relTargetPathForRel(rel, true))
+func (l *genericResource) relPermalinkForRel(rel string, isAbs bool) string {
+	return l.spec.PathSpec.URLizeFilename(l.relTargetPathForRel(rel, false, isAbs, true))
 }
 
-func (l *genericResource) relTargetPathForRel(rel string, isURL bool) string {
+func (l *genericResource) relTargetPathsForRel(rel string) []string {
+	if len(l.baseTargetPathDirs) == 0 {
+		return []string{l.relTargetPathForRelAndBasePath(rel, "", false, false)}
+	}
 
+	var targetPaths = make([]string, len(l.baseTargetPathDirs))
+	for i, dir := range l.baseTargetPathDirs {
+		targetPaths[i] = l.relTargetPathForRelAndBasePath(rel, dir, false, false)
+	}
+	return targetPaths
+}
+
+func (l *genericResource) relTargetPathForRel(rel string, addBaseTargetPath, isAbs, isURL bool) string {
+	if addBaseTargetPath && len(l.baseTargetPathDirs) > 1 {
+		panic("multiple baseTargetPathDirs")
+	}
+	var basePath string
+	if addBaseTargetPath && len(l.baseTargetPathDirs) > 0 {
+		basePath = l.baseTargetPathDirs[0]
+	}
+
+	return l.relTargetPathForRelAndBasePath(rel, basePath, isAbs, isURL)
+}
+
+func (l *genericResource) relTargetPathForRelAndBasePath(rel, basePath string, isAbs, isURL bool) string {
 	if l.targetPathBuilder != nil {
 		rel = l.targetPathBuilder(rel)
 	}
@@ -759,16 +803,19 @@ func (l *genericResource) relTargetPathForRel(rel string, isURL bool) string {
 		rel = path.Join(l.baseURLDir, rel)
 	}
 
-	if !isURL && l.baseTargetPathDir != "" {
-		rel = path.Join(l.baseTargetPathDir, rel)
+	if basePath != "" {
+		rel = path.Join(basePath, rel)
 	}
 
 	if l.baseOffset != "" {
 		rel = path.Join(l.baseOffset, rel)
 	}
 
-	if isURL && l.spec.PathSpec.BasePath != "" {
-		rel = path.Join(l.spec.PathSpec.BasePath, rel)
+	if isURL {
+		bp := l.spec.PathSpec.GetBasePath(!isAbs)
+		if bp != "" {
+			rel = path.Join(bp, rel)
+		}
 	}
 
 	if len(rel) == 0 || rel[0] != '/' {
@@ -787,12 +834,19 @@ func (l *genericResource) String() string {
 }
 
 func (l *genericResource) Publish() error {
-	f, err := l.ReadSeekCloser()
+	fr, err := l.ReadSeekCloser()
 	if err != nil {
 		return err
 	}
-	defer f.Close()
-	return helpers.WriteToDisk(l.targetFilename(), f, l.spec.BaseFs.PublishFs)
+	defer fr.Close()
+	fw, err := helpers.OpenFilesForWriting(l.spec.BaseFs.PublishFs, l.targetFilenames()...)
+	if err != nil {
+		return err
+	}
+	defer fw.Close()
+
+	_, err = io.Copy(fw, fr)
+	return err
 }
 
 // Path is stored with Unix style slashes.
@@ -800,8 +854,12 @@ func (l *genericResource) targetPath() string {
 	return l.relTargetDirFile.path()
 }
 
-func (l *genericResource) targetFilename() string {
-	return filepath.Clean(l.relTargetPath())
+func (l *genericResource) targetFilenames() []string {
+	paths := l.relTargetPaths()
+	for i, p := range paths {
+		paths[i] = filepath.Clean(p)
+	}
+	return paths
 }
 
 // TODO(bep) clean up below
@@ -816,7 +874,7 @@ func (r *Spec) newGenericResource(sourceFs afero.Fs,
 		false,
 		nil,
 		"",
-		"",
+		nil,
 		targetPathBuilder,
 		osFileInfo,
 		sourceFilename,
@@ -831,7 +889,7 @@ func (r *Spec) newGenericResourceWithBase(
 	lazyPublish bool,
 	openReadSeekerCloser OpenReadSeekCloser,
 	urlBaseDir string,
-	targetPathBaseDir string,
+	targetPathBaseDirs []string,
 	targetPathBuilder func(base string) string,
 	osFileInfo os.FileInfo,
 	sourceFilename,
@@ -851,10 +909,10 @@ func (r *Spec) newGenericResourceWithBase(
 	}
 
 	pathDescriptor := resourcePathDescriptor{
-		baseURLDir:        urlBaseDir,
-		baseTargetPathDir: targetPathBaseDir,
-		targetPathBuilder: targetPathBuilder,
-		relTargetDirFile:  dirFile{dir: fpath, file: fname},
+		baseURLDir:         urlBaseDir,
+		baseTargetPathDirs: targetPathBaseDirs,
+		targetPathBuilder:  targetPathBuilder,
+		relTargetDirFile:   dirFile{dir: fpath, file: fname},
 	}
 
 	var po *publishOnce
