@@ -20,17 +20,20 @@ import (
 	"errors"
 	"net/http"
 	"strings"
-	"time"
 
+	"github.com/gohugoio/hugo/cache/filecache"
 	"github.com/gohugoio/hugo/deps"
 	_errors "github.com/pkg/errors"
 )
 
 // New returns a new instance of the data-namespaced template functions.
 func New(deps *deps.Deps) *Namespace {
+
 	return &Namespace{
-		deps:   deps,
-		client: http.DefaultClient,
+		deps:         deps,
+		cacheGetCSV:  deps.FileCaches.GetCSVCache(),
+		cacheGetJSON: deps.FileCaches.GetJSONCache(),
+		client:       http.DefaultClient,
 	}
 }
 
@@ -38,6 +41,9 @@ func New(deps *deps.Deps) *Namespace {
 type Namespace struct {
 	deps *deps.Deps
 
+	cacheGetJSON *filecache.Cache
+	cacheGetCSV  *filecache.Cache
+
 	client *http.Client
 }
 
@@ -48,40 +54,34 @@ type Namespace struct {
 // GetCSV returns nil or a slice slice to use in a short code.
 func (ns *Namespace) GetCSV(sep string, urlParts ...string) (d [][]string, err error) {
 	url := strings.Join(urlParts, "")
+	cache := ns.cacheGetCSV
 
-	var clearCacheSleep = func(i int, u string) {
-		ns.deps.Log.WARN.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
-		time.Sleep(resSleep)
-		deleteCache(url, ns.deps.Fs.Source, ns.deps.Cfg)
-	}
-
-	for i := 0; i <= resRetries; i++ {
-		var req *http.Request
-		req, err = http.NewRequest("GET", url, nil)
-		if err != nil {
-			return nil, _errors.Wrapf(err, "failed to create request for getCSV for resource %s", url)
+	unmarshal := func(b []byte) (error, bool) {
+		if !bytes.Contains(b, []byte(sep)) {
+			return _errors.Errorf("cannot find separator %s in CSV for %s", sep, url), false
 		}
 
-		req.Header.Add("Accept", "text/csv")
-		req.Header.Add("Accept", "text/plain")
-
-		var c []byte
-		c, err = ns.getResource(req)
-		if err != nil {
-			return nil, _errors.Wrapf(err, "failed to read CSV resource %q", url)
-		}
-
-		if !bytes.Contains(c, []byte(sep)) {
-			return nil, _errors.Errorf("cannot find separator %s in CSV for %s", sep, url)
-		}
-
-		if d, err = parseCSV(c, sep); err != nil {
+		if d, err = parseCSV(b, sep); err != nil {
 			err = _errors.Wrapf(err, "failed to parse CSV file %s", url)
 
-			clearCacheSleep(i, url)
-			continue
+			return err, true
 		}
-		break
+
+		return nil, false
+	}
+
+	var req *http.Request
+	req, err = http.NewRequest("GET", url, nil)
+	if err != nil {
+		return nil, _errors.Wrapf(err, "failed to create request for getCSV for resource %s", url)
+	}
+
+	req.Header.Add("Accept", "text/csv")
+	req.Header.Add("Accept", "text/plain")
+
+	err = ns.getResource(cache, unmarshal, req)
+	if err != nil {
+		return nil, _errors.Wrapf(err, "failed to read CSV resource %q", url)
 	}
 
 	return
@@ -90,38 +90,34 @@ func (ns *Namespace) GetCSV(sep string, urlParts ...string) (d [][]string, err e
 // GetJSON expects one or n-parts of a URL to a resource which can either be a local or a remote one.
 // If you provide multiple parts they will be joined together to the final URL.
 // GetJSON returns nil or parsed JSON to use in a short code.
-func (ns *Namespace) GetJSON(urlParts ...string) (v interface{}, err error) {
+func (ns *Namespace) GetJSON(urlParts ...string) (interface{}, error) {
+	var v interface{}
 	url := strings.Join(urlParts, "")
+	cache := ns.cacheGetJSON
 
-	for i := 0; i <= resRetries; i++ {
-		var req *http.Request
-		req, err = http.NewRequest("GET", url, nil)
-		if err != nil {
-			return nil, _errors.Wrapf(err, "Failed to create request for getJSON resource %s", url)
-		}
-
-		req.Header.Add("Accept", "application/json")
-
-		var c []byte
-		c, err = ns.getResource(req)
-		if err != nil {
-			return nil, _errors.Wrapf(err, "failed to get getJSON resource %q", url)
-		}
-		err = json.Unmarshal(c, &v)
-		if err != nil {
-			ns.deps.Log.WARN.Printf("Cannot read JSON from resource %s: %s", url, err)
-			ns.deps.Log.WARN.Printf("Retry #%d for %s and sleeping for %s", i, url, resSleep)
-			time.Sleep(resSleep)
-			deleteCache(url, ns.deps.Fs.Source, ns.deps.Cfg)
-			continue
-		}
-		break
+	req, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return nil, _errors.Wrapf(err, "Failed to create request for getJSON resource %s", url)
 	}
 
+	unmarshal := func(b []byte) (error, bool) {
+		err := json.Unmarshal(b, &v)
+		if err != nil {
+			return err, true
+		}
+		return nil, false
+	}
+
+	req.Header.Add("Accept", "application/json")
+
+	err = ns.getResource(cache, unmarshal, req)
+
 	if err != nil {
 		return nil, _errors.Wrapf(err, "failed to get getJSON resource %q", url)
 	}
-	return
+
+	return v, nil
+
 }
 
 // parseCSV parses bytes of CSV data into a slice slice string or an error
